"""
The `archive.py` module handles scraping archive data from specified URLs.
The retrieval is done by date, with each article on that date being fetched.
"""

import datetime as dt
import dateparser
import requests
import tldextract as url_extract
from lxml.html import HtmlElement, fromstring
from typing import Optional, Any


class Archive:
    """
    A class used to represent an archive of articles for a specific date.

    Attributes
    ----------
    date : dt.date
        The date for which the archive should be fetched.
    doc : Optional[HtmlElement]
        The parsed HTML document of the archive page.

    Methods
    -------
    fetch():
        Fetches the archive page for the specified date.

    parse() -> list[dict[str, Any]]:
        Parses the fetched archive page to extract article details.
    """

    def __init__(self, date: dt.date):
        """
        Constructs the Archive object.

        Parameters
        ----------
        date : dt.date
            The date for which the archive should be fetched.
        """
        self.date: dt.date = date
        self.doc: Optional[HtmlElement] = None

    def fetch(self) -> None:
        """
        Fetches the archive page for the specified date and sets the `doc`
        attribute with the parsed HTML document.

        Raises
        ------
        requests.exceptions.RequestException
            If the HTTP request to fetch the archive page fails.
        """
        archive_url: str = f'https://www.spiegel.de/nachrichtenarchiv/artikel-{self.date.strftime("%d.%m.%Y")}.html'
        response: requests.Response = requests.get(archive_url)
        self.doc: HtmlElement = fromstring(response.text)

    def parse(self) -> list[dict[str, Any]]:
        """
        Parses the fetched archive page to extract details of each article.

        Returns
        -------
        list[dict[str, str]]
            A list of dictionaries, each containing details of an article including URL,
            headline, and whether it's a paid article.

        Raises
        ------
        AssertionError
        """
        assert self.doc is not None, "Fetch the document first before parsing."

        articles: list[dict[str, str]] = []
        for article in self.doc.xpath("//article"):
            article_details: dict[str, Any] = {}

            url: str = article.xpath(".//a/@href")[0]

            if url_extract.extract(url).registered_domain != "spiegel.de":
                continue

            headline: str = article.xpath(".//a/@title")[0]
            spiegel_plus_icon: str = article.xpath(
                'boolean(.//span[@data-flag-name="Spplus-paid"])'
            )
            string_date_published: str = article.xpath("string(./footer/span[1])")
            channel: str = article.xpath("string(./footer/span[3])")

            article_details["url"] = url
            article_details["headline"] = headline
            article_details["is_paid"] = bool(spiegel_plus_icon)
            article_details["date_published"] = self._parse_date(string_date_published)
            article_details["channel"] = channel

            articles.append(article_details)

        return articles

    @staticmethod
    def _parse_date(date_string: str) -> Optional[dt.datetime]:
        """
        Parses a date string using the `dateparser` library.

        Parameters
        ----------
        date_string : str
            The date string to be parsed.

        Returns
        -------
        Optional[dt.datetime]
            The parsed datetime object or None if parsing fails.
        """
        try:
            parsed_date: dt.datetime = dateparser.parse(
                date_string=date_string, languages=["de"], settings={}
            )

            return parsed_date

        except ValueError:
            return None
