"""
The `comments.py` module handles fetching and parsing comments for individual articles.
Comments are extracted based on the article ID, with options for nesting replies.
"""

from collections import defaultdict
import requests
from typing import Optional, Any
from .config import TALK_ENDPOINT_URL


class Comments:
    """A class used to represent and handle comments for a specific article.

    Attributes
    ----------
    article_id : str
        The ID of the article for which comments are to be fetched.
    nesting : bool
        Whether to nest replies under their parent comments or not.
    page_size_limit : int
        The maximum number of comments to fetch per request.
    comments : Optional[list[dict[str, Any]]]
        List to store fetched comments data.

    Methods
    -------
    fetch() -> None:
        Fetches comments from the server and stores them in the `comments` attribute.
    parse() -> list[dict[str, Any]]:
        Parses the fetched comments into a structured format, optionally nesting replies.
    """

    def __init__(
        self, article_id: str, nesting: bool = True, page_size_limit: int = 1000
    ):
        """Constructs a Comments object.

        Parameters
        ----------
        article_id : str
            The ID of the article for which comments are to be fetched.
        nesting : bool, optional
            Whether to nest replies under their parent comments or not (default is True).
        page_size_limit : int, optional
            The maximum number of comments to fetch per request (default is 1000).
        """
        self.article_id: str = article_id
        self.nesting: bool = nesting
        self.page_size_limit: int = page_size_limit
        self.comments: Optional[list] = None

    def fetch(self):
        """Fetches comments from the server and stores them in the `comments` attribute.

        Raises
        ------
        requests.RequestException
            If the request to fetch comments fails.
        """
        query: str = """
            query GetComments($assetId: ID!, $cursor: Cursor, $limit: Int) { 
              asset(id: $assetId) {
                comments(deep: true, query: {sortBy: CREATED_AT, sortOrder: ASC, limit: $limit, cursor: $cursor}) {
                  endCursor
                  hasNextPage
                  nodes {
                    id
                    parent {id}
                    body
                    action_summaries {
                      __typename
                      count
                    }
                    tags {
                      tag {
                        ...
                      }
                      ...
                    }
                    ...
                  }
                }
              }
            }
        """
        self.comments: list = []
        current_cursor = None
        while True:
            response: requests.Response = requests.post(
                TALK_ENDPOINT_URL,
                json={
                    "query": query,
                    "variables": {
                        "assetId": self.article_id,
                        "cursor": current_cursor,
                        "limit": self.page_size_limit,
                    },
                },
            )

            response_data: dict[str, Any] = response.json()["data"]
            comments_page_data: dict[str, Any] = response_data["asset"]["comments"]
            self.comments.extend(comments_page_data["nodes"])
            current_cursor = comments_page_data["endCursor"]
            if not comments_page_data["hasNextPage"]:
                break

    def parse(self) -> list[dict]:
        """Parses the fetched comments into a structured format, optionally nesting replies.

        Returns
        -------
        List[Dict[str, Any]]
            A list of parsed comments, with replies nested under their parent comments if nesting is enabled.

        Raises
        ------
        AssertionError
            If `fetch` method was not called before parsing.
        """
        assert self.comments is not None, "Fetch the comments first before parsing."

        top_level_comments: list[dict[str, Any]] = []
        replies_dict: dict[str, list[dict[str, Any]]] = defaultdict(list)
        for comment in self.comments:
            comment["replies"]: list = []
            parent_ref: Optional[dict[str, Any]] = comment.pop("parent")
            if parent_ref is None:
                top_level_comments.append(comment)
            else:
                if self.nesting:
                    replies_dict[parent_ref["id"]].append(comment)
                else:
                    comment["parent_id"] = parent_ref["id"]
                    top_level_comments.append(comment)

        return top_level_comments
